﻿Public Class ExcelToSql
    Implements IDisposable

    Dim _range As Excel.Range

    Dim _dt As New Data.DataTable

    Public Sub New(ByVal target As Excel.Range)
        _range = target
        PopulateDataTable()
    End Sub

    Private Sub PopulateDataTable()

        Try

            For i As Integer = 1 To _range.Columns.Count

                Dim t = ColumnDataType(_range, i)

                _dt.Columns.Add(New Data.DataColumn(columnName:=CStr(DirectCast(_range.Cells(1, i), Excel.Range).Value)))

                With _dt.Columns(i - 1)
                    .DataType = t
                    .AllowDBNull = True
                End With

                If t Is GetType(String) Then
                    _dt.Columns(_dt.Columns.Count - 1).MaxLength = MaxStringLength(_range, i)
                End If

            Next

            Dim rangeArray = DirectCast(_range.Value, Object(,))

            Dim cols = rangeArray.GetUpperBound(1)
            Dim rows = rangeArray.GetUpperBound(0)

            For i As Integer = 2 To rows
                Dim nr = _dt.NewRow
                For j = 1 To cols
                    If Not rangeArray(i, j) Is Nothing Then
                        nr.Item(j - 1) = rangeArray(i, j)
                    End If
                Next
                _dt.Rows.Add(nr)
            Next

        Catch ex As Exception
            MsgBox(ex.Message)
        End Try

    End Sub

    Private Function ColumnDataType(ByVal target As Excel.Range, ByVal columnNumber As Integer) As Type

        Dim isInteger As Boolean = True
        Dim isDouble As Boolean = True
        Dim isDateTime As Boolean = True

        Dim rangeArray = DirectCast(target.Value, Object(,))

        For i = 2 To rangeArray.GetUpperBound(0)
            Dim j As Integer
            If Integer.TryParse(CStr(rangeArray(i, columnNumber)), j) = False Then
                If Not rangeArray(i, columnNumber) Is Nothing Then
                    isInteger = False
                    Exit For
                End If
            End If
        Next

        If isInteger = False Then
            For i = 2 To rangeArray.GetUpperBound(0)
                Dim j As Double
                If Double.TryParse(CStr(rangeArray(i, columnNumber)), j) = False Then
                    If Not rangeArray(i, columnNumber) Is Nothing Then
                        isDouble = False
                        Exit For
                    End If
                End If
            Next
        End If

        If isInteger = False AndAlso isDouble = False Then
            For i = 2 To rangeArray.GetUpperBound(0)
                Dim j As DateTime
                If DateTime.TryParse(CStr(rangeArray(i, columnNumber)), j) = False Then
                    If Not rangeArray(i, columnNumber) Is Nothing Then
                        isDateTime = False
                        Exit For
                    End If
                End If
            Next
        End If

        rangeArray = Nothing

        If isInteger Then
            Return GetType(Integer)
        ElseIf isDouble = True Then
            Return GetType(Double)
        ElseIf isDateTime = True Then
            Return GetType(Date)
        Else
            Return GetType(String)
        End If

    End Function

    Private Function MaxStringLength(ByVal target As Excel.Range, ByVal columnNumber As Integer) As Integer

        Dim x = DirectCast(target.Value, Object(,))

        Dim maxLength As Integer = 1

        For i = 2 To x.GetUpperBound(0)
            If Not x(i, columnNumber) Is Nothing Then
                Dim sl = CStr(x(i, columnNumber)).Length
                If sl > maxLength Then
                    maxLength = sl
                End If
            End If
        Next

        Return maxLength

    End Function

    Public ReadOnly Property DataTable() As Data.DataTable
        Get
            Return _dt
        End Get
    End Property

    Public ReadOnly Property Script() As String
        Get

            Dim columnList As New List(Of String)
            For i As Integer = 0 To _dt.Columns.Count - 1
                Dim columnType = _dt.Columns(i).DataType
                If columnType Is GetType(Integer) Then
                    columnList.Add(String.Format("[{0}] int", _dt.Columns(i).ColumnName))
                ElseIf columnType Is GetType(Double) Then
                    columnList.Add(String.Format("[{0}] float", _dt.Columns(i).ColumnName))
                ElseIf columnType Is GetType(DateTime) Then
                    columnList.Add(String.Format("[{0}] datetime", _dt.Columns(i).ColumnName))
                ElseIf columnType Is GetType(String) Then
                    columnList.Add(String.Format("[{0}] varchar({1})", _dt.Columns(i).ColumnName, _dt.Columns(i).MaxLength))
                Else
                    Throw New Exception("Unsupported Type")
                End If
            Next

            Dim sb As New System.Text.StringBuilder

            Dim quote As Func(Of String, String) = Function(s) String.Format("'{0}'", s.Replace("'", "''"))

            sb.Append(String.Format("create table #t1({0})", columnList.Aggregate(Function(x, y) x & Chr(10) & "," & y)))

            Dim columnCount = _dt.Columns.Count
            sb.Append(vbCrLf)
            For i As Integer = 0 To _dt.Rows.Count - 1
                sb.Append(" insert #t1 values (")
                For j = 0 To columnCount - 1
                    If TypeOf _dt.Rows(i).Item(j) Is DBNull Then
                        sb.Append("NULL")
                        If j <> columnCount - 1 Then
                            sb.Append(",")
                        End If
                    Else
                        If _dt.Columns(j).DataType Is GetType(String) Or _dt.Columns(j).DataType Is GetType(DateTime) Then
                            If _dt.Rows(i).Item(j).ToString.ToUpper = "NULL" Then
                                sb.Append("NULL")
                            Else
                                sb.Append(quote(CStr(_dt.Rows(i).Item(j))))
                            End If
                        Else
                            sb.Append(_dt.Rows(i).Item(j))
                        End If
                        If j <> columnCount - 1 Then
                            sb.Append(",")
                        End If
                    End If
                Next

                sb.Append(")")
                sb.Append(vbCrLf)

            Next

            sb.Append("select * from #t1")

            Return sb.ToString

        End Get
    End Property

#Region "IDisposable Support"

    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                _dt = Nothing
            End If

            ' TODO: free unmanaged resources (unmanaged objects) and override Finalize() below.
        End If
        Me.disposedValue = True
    End Sub

    ' This code added by Visual Basic to correctly implement the disposable pattern.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

#End Region

End Class
